@@ -177,6 +177,7 @@ sub close_session {
 #       defaults to 3 minutes
 #    active: seconds for timeout of sockets used in sessions, defaults to
 #       30 seconds
+#    closed: seconds for timeout of sockets from closed sessions, default 1 sec
 # Returns: @expired
 #   @expired: list of infos about expired sessions using sessions info_as_hash
 ############################################################################
@@ -187,7 +188,8 @@ sub expire {
     $args{time}   ||= gettimeofday();
     $args{unused} ||= 3*60; # unused sockets after 3 minutes
     $args{active} ||= 30;   # active sessions after 30 seconds
-    DEBUG( 100,"expire now=$args{time} unused=$args{unused} active=$args{active}" );
+    $args{closed} ||= 1;    # closed sessions after 1 seconds
+    DEBUG( 100,"expire now=$args{time} unused=$args{unused} active=$args{active} closed=$args{closed}" );
     my @expired;
     my $calls = $self->{calls};
     foreach my $callid ( keys %$calls ) {
@@ -341,7 +343,7 @@ sub unget_rtp_sockets {
 ############################################################################
 
 package Net::SIP::NATHelper::Call;
-use fields qw( callid from );
+use fields qw( callid from expired );
 use Hash::Util 'lock_keys';
 use List::Util 'max';
 use Net::SIP::Debug;
@@ -445,6 +447,10 @@ sub activate_session {
     my $sess = $sessions->{$idto} = $self->create_session( $gfrom,$gto,$param );
     DEBUG( 10,"new session {$sess->{id}} $self->{callid},$cseq $idfrom -> $idto" );
 
+    # expire the now unused previous sessions in this call immediately
+    # will be returned at the next call to Base::expire
+    $self->{expired} = [ $self->expire() ];
+
     return ( $sess->info_as_hash( $self->{callid},$cseq ), 0 );
 }
 
@@ -537,10 +543,12 @@ sub expire {
     my Net::SIP::NATHelper::Call $self = shift;
     my %args = @_;
 
-    my $expire_unused = $args{time} - $args{unused};
-    my $expire_active = $args{time} - $args{active};
+    my $expire_unused = $args{unused} ? $args{time} - $args{unused} : 0;
+    my $expire_active = $args{active} ? $args{time} - $args{active} : 0;
+    my $expire_closed = $args{closed} ? $args{time} - $args{closed} : 0;
 
-    my @expired;
+    my @expired = @{$self->{expired} || []}; # from previous calls inside activate_session
+    @{$self->{expired}} = ();
     my %active_pairs; # mapping [idfrom,idto]|[idto,idfrom] -> session.created
     my $need_next_pass;
     my $by_from = $self->{from};
@@ -608,8 +616,8 @@ sub expire {
 			    DEBUG( 10,"$self->{callid} expired socketgroup $v->{id} because created($v->{created}) < unused($expire_unused)" );
 			    $expired_sg{$v} = 1;
 			}
-		    } elsif ( $lastmod < $expire_active ) {
-			DEBUG( 10,"$self->{callid} expired socketgroup $v->{id} because lastmod($lastmod) < active($expire_active)" );
+		    } elsif ( $lastmod < $expire_closed ) {
+			DEBUG( 10,"$self->{callid} expired socketgroup $v->{id} because lastmod($lastmod) < closed($expire_closed)" );
 			$expired_sg{$v} = 1;
 		    }
 		}
